# COPYRIGHT 2018 ESRI
#
# TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
# Unpublished material - all rights reserved under the
# Copyright Laws of the United States.
#
# For additional information, contact:
# Environmental Systems Research Institute, Inc.
# Attn: Contracts Dept
# 380 New York Street
# Redlands, California, USA 92373
#
# email: contracts@esri.com

import arcgisscripting
from .util import get_geometry, convert_string_to_date_time
import numbers
import arcpy
from arcpy.ia import Apply, Merge
from collections import defaultdict
import warnings
from datetime import datetime


class _RasterCollectionCursor:
    def __init__(self, data_frame):
        self._data_frame = data_frame
        self._index = 0

    def __len__(self):
        return self._data_frame.shape[0]

    def __iter__(self):
        return self

    def __next__(self):
        if self._index >= len(self):
            raise StopIteration

        row = self._data_frame.iloc[self._index]
        self._index += 1

        # convert pandas Series to native python dictionary^M
        return row.to_dict()


class RasterCollection:
    def __init__(self, rasters, attribute_dict=None, where_clause=None, query_geometry=None):
        self._is_mosaic = True
        self._df = None
        self._raster_collection = None
        import pandas as pd

        if isinstance(rasters, arcgisscripting._ia.RasterCollection):
            self._raster_collection = rasters
        elif isinstance(rasters, str):
            if attribute_dict is not None:
                raise ValueError('attribute_dict must be None for mosaic/multidimensional raster input')
            self._raster_collection = arcgisscripting._ia.RasterCollection(rasters, where_clause, query_geometry)
        elif isinstance(rasters, pd.DataFrame):
            self._is_mosaic = False
            self._df = rasters.copy()
        elif isinstance(rasters, list):
            for i, raster in enumerate(rasters):
                if not isinstance(raster, arcpy.Raster):
                    rasters[i] = arcpy.Raster(raster)

            in_raster_collection = {'Raster': rasters}
            self._is_mosaic = False

            if attribute_dict is not None and isinstance(attribute_dict, dict):
                if 'Raster' in attribute_dict:
                    raise ValueError('attribute_dict should not use "Raster" as the key')
                in_raster_collection.update(attribute_dict)
            self._df = pd.DataFrame(in_raster_collection)


        if self._raster_collection is None:
            df_copy = self._df
            df_rasters = {'rasters': df_copy['Raster'].to_list()}
            attribute_table = df_copy.to_dict(orient='list')
            df_rasters.update(attribute_table)
            self._raster_collection = arcgisscripting._ia.RasterCollection(df_rasters)

    def _export_data_frame(self):
        import pandas as pd
        if self._df is not None:
            return self._df

        data = {}
        for field in self.fields:
            try:
                data[field] = self.getFieldValues(field)
            except:
                continue
        return pd.DataFrame(data=data)

    def _build_query_string(self, field_name, operator, field_values):
        operator_map = {
            "equals": "=",
            "less_than": "<",
            "greater_than": ">",
            "not_equals": "<>",
            "not_less_than": ">=",
            "not_greater_than": "<=",
        }

        if operator in operator_map:
            if isinstance(field_values, numbers.Number):
                return field_name + ' ' + operator_map[operator] + ' ' + str(field_values)
            elif isinstance(field_values, str):
                return field_name + ' ' + operator_map[operator] + ' \'' + field_values + '\''
            else:
                raise TypeError('field_value must be numeric or string')

        elif operator in ['starts_with', 'ends_with', 'not_starts_with', 'not_ends_with', 'contains', 'not_contains']:
            if not isinstance(field_values, str):
                raise TypeError('field_value must be string')
            if operator == 'starts_with':
                return field_name + ' LIKE ' + '\'' + field_values + '%\''
            elif operator == 'ends_with':
                return field_name + ' LIKE' + '\'%' + field_values + '\''
            elif operator == 'not_starts_with':
                return field_name + ' NOT LIKE ' + '\'' + field_values + '%\''
            elif operator == 'not_ends_with':
                return field_name + ' NOT LIKE ' + '\'%' + field_values + '\''
            elif operator == 'contains':
                return field_name + ' LIKE ' + '\'%' + field_values + '%\''
            elif operator == 'not_contains':
                return field_name + ' NOT LIKE ' + '\'%' + field_values + '%\''
        elif operator == 'in':
            if not isinstance(field_values, list):
                raise TypeError('field_values must be type list for operator "in"')
            values = '('
            for item in field_values:
                if not (isinstance(item, numbers.Number) or isinstance(item, str)):
                    raise TypeError('item in field_values must be numeric or string')
                if values == '(':
                    values += '\'' + item + '\'' if isinstance(item, str) else str(item)
                else:
                    values += ',\'' + item + '\'' if isinstance(item, str) else ',' + str(item)
            values += ')'
            return field_name + ' IN ' + values
        elif operator == 'not_in':
            values = '('
            for item in field_values:
                if not (isinstance(item, numbers.Number) or isinstance(item, str)):
                    raise TypeError('item in field_values must be numeric or string')
                if values == '(':
                    values += '\'' + item + '\'' if isinstance(item, str) else str(item)
                else:
                    values += ',\'' + item + '\'' if isinstance(item, str) else ',' + str(item)
            values += ')'
            return field_name + ' NOT IN ' + values
        else:
            raise ValueError('invalid operator value')

    def _aggregate_function(self, fn_name, extent_type=None, cellsize_type=None):
        if 'Raster' in self.fields:
            field_value = 'Raster'
        elif 'Path' in self.fields:
            field_value = 'Path'
        else:
            raise RuntimeError('invalid raster collection')

        extent_types = {
            "FIRSTOF": 0,
            "INTERSECTIONOF": 1,
            "UNIONOF": 2,
            "LASTOF": 3
        }

        cellsize_types = {
            "FIRSTOF": 0,
            "MINOF": 1,
            "MAXOF": 2,
            "MEANOF": 3,
            "LASTOF": 4
        }
        fn_args_dict = {}

        if extent_type is not None:
            in_extent_type = extent_types[extent_type.upper()]
            if in_extent_type is not None:
                fn_args_dict['ExtentType'] = in_extent_type
        
        if cellsize_type is not None:
            in_cellsize_type = cellsize_types[cellsize_type.upper()]
            if in_cellsize_type is not None:
                fn_args_dict['CellsizeType'] = in_cellsize_type
        
        return Apply(self, fn_name, fn_args_dict)

    def filter(self, where_clause='', query_geometry_or_extent=None, raster_query=''):
        """
        filter a raster collection based on attribute and/or spatial queries

        :param where_clause (SQL Expression): An SQL expression used to select a subset of rasters
        :param query_geometry_or_extent (feature, feature layer, raster dataset, raster layer, extent, geometry): items in the collection that fails to intersect the given geometry will be excluded
        :param raster_query (SQL Expression): An SQL expression used to select a subset of rasters by the raster's key properties
        :return(RaterCollection): a raster collection that only contains items sastisfying the queries
        """
        if not isinstance(where_clause, str):
            raise TypeError("where_clause must be string type")

        if self._is_mosaic:
            return RasterCollection(self._raster_collection.filter(None if not where_clause else where_clause,
                                                                   get_geometry(query_geometry_or_extent),
                                                                   None if not raster_query else raster_query))
        else:
            if "LIKE" in where_clause or "NOT LIKE" in where_clause:
                raise RuntimeError("Local RasterCollection does not support LIKE or NOT LIKE in where_clause.")
            new_raster_collection = RasterCollection(self._df)
            if query_geometry_or_extent is not None:
                new_raster_collection = new_raster_collection.filterByGeometry(query_geometry_or_extent)
            if where_clause:
                new_raster_collection._df.query(where_clause, inplace=True)
            return new_raster_collection

    @property
    def fields(self):
        if not self._is_mosaic:
            return self._df.columns.to_list()
        return list(self._raster_collection.fields) # otherwise it is tuple

    @property
    def count(self):
        if not self._is_mosaic:
            return self._df.shape[0]
        return self._raster_collection.count

    def __iter__(self):
        if not self._is_mosaic:
            return _RasterCollectionCursor(self._df)
        return iter(self._raster_collection)

    def __len__(self):
        return self.count

    def __getitem__(self, item):
        if not self._is_mosaic:
            return self._df.iloc[item].to_dict()
        return self._raster_collection[item]

    def _repr_html_(self):
        df = self._export_data_frame()
        str_frmt = lambda x: str(x)
        frmt = {col:str_frmt for col in df.columns}
        return df.to_html(formatters=frmt, notebook=True)

    def filterByTime(self, start_time="", end_time="", time_field_name="StdTime", date_time_format=None):
        """
        Shortcut to filter a collection by time

        :param start_time(String): a string representation of the start time, format must be '%m/%d/%Y', i.e., '10/03/2019'
        :param end_time(String): a string representation of the end time. format must be '%m/%d/%Y', i.e., '10/03/2019'.
                                This is optional. If not provided, only items that have exactly same time value as the start_time would be inlucded in the output
        :param time_field_name(String): the name of the field containing the time information for each item. Default: "StdTime"
        :param date_time_field(String): default is None which uses the default string representation of arcgis date field type
                                        if default is used, the date_time_format means '%Y-%m-%dT%H:%M:%S'
        :param date_time_format(String): the time format that is used to format the time field values. Please ref the python
                                date time standard for this argument. https://docs.python.org/3/library/datetime.html#strftime-and-strptime-behavior
                                Default is None and this means using the Pro standard time format '%Y-%m-%dT%H:%M:%S'
                                and ignoring the following sub-second.

        :return(RaterCollection): a filtered raster collection.
        """
        # type checking
        if not start_time and not end_time:
            return self

        if not isinstance(start_time, str):
            raise TypeError("start_time should be string or None.")
        if not isinstance(end_time, str):
            raise TypeError("end_time should be string or None.")

        if time_field_name not in self.fields:
            raise ValueError('the time_field_name does not exist. Please input a valid name for time field.')

        if start_time:
            start_time = convert_string_to_date_time(start_time, date_time_format)
        if end_time:
            end_time = convert_string_to_date_time(end_time, date_time_format)

        if self._is_mosaic:

            # the time format used in SQL is month/day/year hour:min:seconds
            if start_time:
                start_time_stamp = str(start_time.month)+"/"+str(start_time.day)+"/" + str(start_time.year) + " "+str(start_time.hour)+":"+str(start_time.minute)+":"+str(start_time.second)
                sql_query1 = time_field_name + ' >= timestamp \'' + start_time_stamp + '\''
            else:
                sql_query1 = ''
            if end_time:
                end_time_stamp = str(end_time.month)+"/"+str(end_time.day)+"/" + str(end_time.year) + " "+str(end_time.hour)+":"+str(end_time.minute)+":"+str(end_time.second)
                sql_query2 = time_field_name + ' <= timestamp \'' + end_time_stamp + '\''
            else:
                sql_query2 = ''

            if sql_query1 and sql_query2:
                sql_query = sql_query1 + ' AND ' + sql_query2
            else:
                sql_query = sql_query1 if sql_query1 else sql_query2

            return self.filter(sql_query)
        else:
            # RasterCollection is created from a list of rasters


            filtered_rasters = []
            attribute_dict = defaultdict(list)

            drop_tzinfo=True
            if start_time and start_time.tzinfo:
                drop_tzinfo=False

            if end_time and end_time.tzinfo:
                drop_tzinfo=False

            for item in iter(self):
                raster = self._get_raster_from_item(item)

                date_time = convert_string_to_date_time(item[time_field_name], date_time_format, drop_tzinfo)

                if start_time and date_time < start_time:
                    continue
                elif end_time and date_time > end_time:
                    continue
                else:
                    filtered_rasters.append(raster)
                    for key, value in item.items():
                        if key not in ['Raster', 'Path']:
                            attribute_dict[key].append(value)

            if not filtered_rasters:
                warnings.warn(
                    "Warning: the output is None because no items have raster properties satisfying the query")

            return RasterCollection(filtered_rasters, attribute_dict) if filtered_rasters else None

    def filterByGeometry(self, query_geometry_or_extent=None):
        """
        Shortcut to filter a collection by geometries. Only those items that have intersection with the given geometry would be kept in the output

        :param query_geometry_or_extent (feature, feature layer, raster dataset, raster layer, extent, geometry): items in the collection that fails to intersect the given geometry will be excluded
        :return(RaterCollection): a filtered raster collection
        """
        if not self._is_mosaic:
            filter_geometry = get_geometry(query_geometry_or_extent)
            filtered_rasters = []
            attribute_dict = defaultdict(list)

            for item in iter(self):
                raster = self._get_raster_from_item(item)
                if not filter_geometry.disjoint(raster.extent):
                    filtered_rasters.append(raster)
                    for key, value in item.items():
                        if key not in ['Raster', 'Path']:
                            attribute_dict[key].append(value)

            if not filtered_rasters:
                warnings.warn(
                    "Warning: the output is None because no items have raster properties satisfying the query")
            return RasterCollection(filtered_rasters, attribute_dict) if filtered_rasters else None

        return self.filter(query_geometry_or_extent=query_geometry_or_extent)

    def filterByAttribute(self, field_name, operator, field_values):
        """
        Shortcut to filter a collection by attribute

        :param field_name(String): the name of the field to filter
        :param operator(String): one of "EQUALS", "LESS_THAN", "GREATER_THAN","NOT_EQUALS", "NOT_LESS_THAN", "NOT_GREATER_THAN",
                        "STARTS_WITH", "ENDS_WITH", "NOT_STARTS_WITH", "NOT_ENDS_WITH", "CONTAINS", "NOT_CONTAINS",
                        "IN" and "NOT_IN"
        :param field_values (String, Numeric, List): the values to compare against.
        :return(RaterCollection): a filtered raster collection
        """
        if not isinstance(field_name, str):
            raise TypeError("field_name should be string")

        if field_name not in self.fields:
            raise ValueError('field_name does not exist')

        def filter_by_attribute_for_list_rasters(field_name, operator, field_values):
            filtered_rasters = []
            attribute_dict = defaultdict(list)

            for item in iter(self):
                raster = self._get_raster_from_item(item)

                field_value = item[field_name]

                selected = False
                if operator == 'equals':
                    selected = (field_value == field_values)
                elif operator == 'less_than':
                    selected = (field_value < field_values)
                elif operator == 'greater_than':
                    selected = (field_value > field_values)
                elif operator == 'not_equals':
                    selected = (field_value != field_values)
                elif operator == 'not_less_than':
                    selected = (field_value >= field_values)
                elif operator == 'not_greater_than':
                    selected = (field_value <= field_values)
                elif operator == 'starts_with':
                    selected = (field_value.startswith(field_values))
                elif operator == 'ends_with':
                    selected = (field_value.endswith(field_values))
                elif operator == 'not_starts_with':
                    selected = (not field_value.startswith(field_values))
                elif operator == 'not_ends_with':
                    selected = (not field_value.endswith(field_values))
                elif operator == 'contains':
                    selected = (field_values in field_value)
                elif operator == 'not_contains':
                    selected = (field_values not in field_value)
                elif operator == 'in':
                    if not isinstance(field_values, list):
                        raise TypeError(
                            "Invalid field_values. Notet that field_values must be a list when operator is 'in'")
                    selected = (field_value in field_values)
                elif operator == 'not_in':
                    if not isinstance(field_values, list):
                        raise TypeError(
                            "Invalid field_values. Notet that field_values must be a list when operator is 'not_in'")
                    selected = (field_value not in field_values)
                else:
                    raise ValueError('invalid operator')

                if selected:
                    filtered_rasters.append(raster)
                    for key, value in item.items():
                        if key not in ['Raster', 'Path']:
                            attribute_dict[key].append(value)

            if not filtered_rasters:
                warnings.warn(
                    "Warning: the output is None because no items have raster properties satisfying the query")
            return RasterCollection(filtered_rasters, attribute_dict) if filtered_rasters else None

        if self._is_mosaic:
            query_string = self._build_query_string(field_name, operator.lower(), field_values)
            return self.filter(query_string)
        else:
            return filter_by_attribute_for_list_rasters(field_name, operator.lower(), field_values)

    def sort(self, field_name, ascending=True):
        """
        sort a raster collection by the specified field name

        :param field_name (String): the field to sort by
        :param ascending (Bool): Whether to sort in ascending or descending order. Default: true (ascending)
        :return(RaterCollection): a sorted raster collection
        """
        if not isinstance(field_name, str):
            raise TypeError('invalid field_name type')

        if field_name not in self.fields:
            raise ValueError('field_name does not exist')

        if ascending not in [True, False]:
            raise ValueError('invalid ascending parameter')

        if self._is_mosaic:
            return RasterCollection(self._raster_collection.sort(field_name, ascending))
        else:
            # raise RuntimeError("Sort operation is not supported for raster collection created from a list of rasters")
            sorted_data_frame = self._export_data_frame().sort_values(by=field_name, ascending=ascending)
            # rasters = sorted_data_frame.pop('Raster').to_list()
            return RasterCollection(sorted_data_frame)

    def getFieldValues(self, field_name, max_count=0):
        """
        get values of a given field
        :param field_name (String): the field name to extract values from
        :param max_count (Int): the number to limit the collection to.
        :return(RaterCollection): a list of values at the given field
        """
        if not isinstance(max_count, int):
            raise TypeError("max_count must be positive integer")
        if max_count < 0:
            raise ValueError("max_count must be positive integer")

        if not isinstance(field_name, str):
            raise TypeError("field_name must be string type")
        if field_name not in self.fields:
            raise ValueError('field_name does not exist')

        if max_count == 0:
            max_count = self.count

        if not self._is_mosaic:
            return self._df[field_name].tolist()[:max_count]

        return self._raster_collection.values(field_name, max_count)

    def toMultidimensionalRaster(self, variable_field_name='', dimension_field_names=''):
        """
        convert the raster collection to a multidimensional raster

        :param variable_field_name (String): the name of the field that contains the variable names
        :param dimension_field_names (String, List of String): the name(s) of the fields that contains the dimension names
        :return(Raster): a multidimensional raster. Each item in the source raster collection becomes a slice in the multidimensional raster
        """
        if not isinstance(variable_field_name, str):
            raise TypeError('variable_field_name must be string type')
        if not (isinstance(dimension_field_names, list) or isinstance(dimension_field_names, str)):
            raise TypeError('dimension_field_names must be a single string or a list of string')

        if variable_field_name == '' and dimension_field_names == '':
            return self._raster_collection.raster()

        if variable_field_name not in self.fields:
            raise ValueError('variable_field_name does not exist')

        if isinstance(dimension_field_names, list):
            for dimension_field_name in dimension_field_names:
                if dimension_field_name not in self.fields:
                    raise ValueError('the given dimension_field_name does not exist')
            dimension_field_names = ','.join(dimension_field_names)
        else:
            if dimension_field_names not in self.fields:
                raise ValueError('the given dimension_field_name does not exist')

        return self._raster_collection.raster(True, variable_field_name, dimension_field_names)

    def max(self, ignore_nodata=True, extent_type="FirstOf", cellsize_type="FirstOf"):
        """
        For each band, calculate the maximum value of each pixel across the the stack of rasters. Bands are matched by band index

        :param ignore_nodata (Bool): Specifies whether NoData values are ignored.

                                    - True : The method will include all valid pixels and ignore any NoData pixels.
                                                This is the default.
                                    - False : The method will result in NoData if there are any NoData values.
        :param extent_type(string): one of "FirstOf", "IntersectionOf" "UnionOf", "LastOf"
        :param cellsize_type(string): one of "FirstOf", "MinOf", "MaxOf "MeanOf", "LastOf"
        :return(Raster): a raster that contains the maximum value of each pixel at each band
        """
        fn = "MaxContinueOnNoData" if ignore_nodata else "Max"
        return self._aggregate_function(fn_name=fn, extent_type=extent_type, cellsize_type=cellsize_type)

    def min(self, ignore_nodata=True, extent_type="FirstOf", cellsize_type="FirstOf"):
        """
        For each band, calculate the minimum value of each pixel across the the stack of rasters. Bands are matched by band index
        :param ignore_nodata (Bool): Specifies whether NoData values are ignored.

                                    - True : The method will include all valid pixels and ignore any NoData pixels.
                                                This is the default.
                                    - False : The method will result in NoData if there are any NoData values.
        :param extent_type(string): one of "FirstOf", "IntersectionOf" "UnionOf", "LastOf"
        :param cellsize_type(string): one of "FirstOf", "MinOf", "MaxOf "MeanOf", "LastOf"
        :return(Raster): a raster that contains the minimum value of each pixel at each band
        """
        fn = "MinContinueOnNoData" if ignore_nodata else "Min"
        return self._aggregate_function(fn_name=fn, extent_type=extent_type, cellsize_type=cellsize_type)
        

    def median(self, ignore_nodata=True, extent_type="FirstOf", cellsize_type="FirstOf"):
        """
        For each band, calculate the median value of each pixel across the the stack of rasters. Bands are matched by band index

        :param ignore_nodata (Bool): Specifies whether NoData values are ignored.

                                    - True : The method will include all valid pixels and ignore any NoData pixels.
                                                This is the default.
                                    - False : The method will result in NoData if there are any NoData values.
        :param extent_type(string): one of "FirstOf", "IntersectionOf" "UnionOf", "LastOf"
        :param cellsize_type(string): one of "FirstOf", "MinOf", "MaxOf "MeanOf", "LastOf"
        :return(Raster): a raster that contains the median value of each pixel at each band
        """
        fn = "MedContinueOnNoData" if ignore_nodata else "Med"
        return self._aggregate_function(fn_name=fn, extent_type=extent_type, cellsize_type=cellsize_type)

    def mean(self, ignore_nodata=True, extent_type="FirstOf", cellsize_type="FirstOf" ):
        """
        For each band, calculate the mean value of each pixel across the the stack of rasters. Bands are matched by band index

        :param ignore_nodata (Bool): Specifies whether NoData values are ignored.

                                    - True : The method will include all valid pixels and ignore any NoData pixels.
                                                This is the default.
                                    - False : The method will result in NoData if there are any NoData values.
        :param extent_type(string): one of "FirstOf", "IntersectionOf" "UnionOf", "LastOf"
        :param cellsize_type(string): one of "FirstOf", "MinOf", "MaxOf "MeanOf", "LastOf"
        :return(Raster): a raster that contains the mean value of each pixel at each band
        """
        fn = "MeanContinueOnNoData" if ignore_nodata else "Mean"
        return self._aggregate_function(fn_name=fn, extent_type=extent_type, cellsize_type=cellsize_type)

    def majority(self, ignore_nodata=True, extent_type="FirstOf", cellsize_type="FirstOf"):
        """
        For each band, calculate the majority value of each pixel across the the stack of rasters. Bands are matched by band index

        :param ignore_nodata (Bool): Specifies whether NoData values are ignored.

                                    - True : The method will include all valid pixels and ignore any NoData pixels.
                                                This is the default.
                                    - False : The method will result in NoData if there are any NoData values.
        :param extent_type(string): one of "FirstOf", "IntersectionOf" "UnionOf", "LastOf"
        :param cellsize_type(string): one of "FirstOf", "MinOf", "MaxOf "MeanOf", "LastOf"
        :return(Raster): a raster that contains the majority value of each pixel at each band
        """
        fn = "MajorityContinueOnNoData" if ignore_nodata else "Majority"
        return self._aggregate_function(fn_name=fn, extent_type=extent_type, cellsize_type=cellsize_type)

    def sum(self, ignore_nodata=True, extent_type="FirstOf", cellsize_type="FirstOf"):
        """
        For each band, calculate the sum value of each pixel across the the stack of rasters. Bands are matched by band index

        :param ignore_nodata (Bool): Specifies whether NoData values are ignored.

                                    - True : The method will include all valid pixels and ignore any NoData pixels.
                                                This is the default.
                                    - False : The method will result in NoData if there are any NoData values.
        :param extent_type(string): one of "FirstOf", "IntersectionOf" "UnionOf", "LastOf"
        :param cellsize_type(string): one of "FirstOf", "MinOf", "MaxOf "MeanOf", "LastOf"
        :return(Raster): a raster that contains the sum value of each pixel at each band
        """
        fn = "SumContinueOnNoData" if ignore_nodata else "Sum"
        return self._aggregate_function(fn_name=fn, extent_type=extent_type, cellsize_type=cellsize_type)

    def std(self, ignore_nodata=True, extent_type="FirstOf", cellsize_type="FirstOf"):
        """
        For each band, calculate standard deviation of each pixel across the the stack of rasters. Bands are matched by band index

        :param ignore_nodata (Bool): Specifies whether NoData values are ignored.

                                    - True : The method will include all valid pixels and ignore any NoData pixels.
                                                This is the default.
                                    - False : The method will result in NoData if there are any NoData values.
        :param extent_type(string): one of "FirstOf", "IntersectionOf" "UnionOf", "LastOf"
        :param cellsize_type(string): one of "FirstOf", "MinOf", "MaxOf "MeanOf", "LastOf"
        :return(Raster): a raster that contains the sum value of each pixel at each band
        """
        fn = "StdContinueOnNoData" if ignore_nodata else "Std"
        return self._aggregate_function(fn_name=fn, extent_type=extent_type, cellsize_type=cellsize_type)

    def mosaic(self, mosaic_method="FIRST"):
        """
        Composites all the rasters in a collection to a single raster

        :param mosaic_method (String): the method used to handle overlapping areas, including FIRST(default),'LAST', 'MIN', 'MAX', 'MEAN', 'MEDIAN', 'SUM'
        :return(Raster): a raster after mosaicing all the rasters in the collection
        """
        if not isinstance(mosaic_method, str) or mosaic_method.upper() not in ['FIRST', 'LAST', 'MIN', 'MAX', 'MEAN',
                                                                               'MEDIAN', 'SUM']:
            raise ValueError("invalid mosic method")

        mosaic_method = mosaic_method.upper()
        if mosaic_method in ['FIRST', 'LAST']:
            if 'Raster' in self.fields:
                return Merge(self.getFieldValues('Raster'), resolve_overlap=mosaic_method)
            elif 'Path' in self.fields:
                rasters = [arcpy.Raster(path) for path in self.getFieldValues('Path')]
                return Merge(rasters, resolve_overlap=mosaic_method)
            else:
                raise ValueError('invalid raster collection')
        else:
            function_map = {"MIN":"MinContinueOnNoData",
                            "MAX":"MaxContinueOnNoData",
                            "MEAN":"MeanContinueOnNoData",
                            "MEDIAN":"MedContinueOnNoData",
                            "SUM":"SumContinueOnNoData"}
                            
            if mosaic_method.upper() in function_map.keys():
                return self._aggregate_function(fn_name=function_map[mosaic_method.upper()], extent_type='UnionOf') #ExtentType 2 - Union of extents
            else:
                raise RuntimeError('invalid mosaic method ' + str(mosaic_method))

    def qualityMosaic(self, quality_rc_or_list, statistic_type="MAX"):
        """
        Composites all the rasters in a collection based on a quality input. By default, each pixel in the output is based on which image in the collection has the same index with the image in the quality_rc_or_list that has the maximum value (default)

        :param quality_rc_or_list(RasterCollection or List): the raster colltion or a list of Raster to be used as quality. It must have same number of items with the collection to be mosaiced.
        :param statistic_type(String): the statistic method used to get the index from the quality rc. Its value could be MAX(default),MIN, MEDIAN
        :return(Raster): a raster after mosaicing all the rasters in the collection
        """
        if isinstance(quality_rc_or_list, list):
            for item in quality_rc_or_list:
                if not isinstance(item, arcpy.Raster):
                    raise TypeError('quality_rc must be a RasterCollection or a list of Raster objects')
        elif not isinstance(quality_rc_or_list, arcpy.ia.RasterCollection):
            raise TypeError('quality_rc must be a RasterCollection or a list of Raster objects')

        if not isinstance(statistic_type, str) or statistic_type.upper() not in ['MAX', 'MIN', 'MEDIAN']:
            raise ValueError('invalid statistic_type value')

        statistic_type = statistic_type.upper()

        statistics_type_code = {'MAX': 0, 'MIN': 1, 'MEDIAN': 2}

        if isinstance(quality_rc_or_list, arcpy.ia.RasterCollection):
            if quality_rc_or_list.count != self.count:
                raise ValueError('the quality collection must have same number of items as the calling collection')

            if 'Raster' in quality_rc_or_list.fields:
                rasters = quality_rc_or_list.getFieldValues('Raster')
            elif 'Path' in quality_rc_or_list.fields:
                rasters = [arcpy.Raster(path) for path in quality_rc_or_list.getFieldValues('Path')]
        elif isinstance(quality_rc_or_list, list):
            if len(quality_rc_or_list) != self.count:
                raise ValueError('the quality raster list must have same number of items as the calling collection')
            rasters = quality_rc_or_list
        else:
            raise ValueError('invalid quality_rc parameter')

        arg_statistics_result = Apply(rasters, 'ArgStatistics',
                                      {'ArgStatisticsType': statistics_type_code[statistic_type], 'IgnoreNoData': True, "ExtentType":2})

        arg_statistics_result = arg_statistics_result + 1  # the index in argstatistics output counts from 0, but Pick counts from 1

        if 'Raster' in self.fields:
            rasters = self.getFieldValues('Raster')
        elif 'Path' in self.fields:
            rasters = self.getFieldValues('Path')
        else:
            raise ValueError('invalid raster collection')

        return Apply([arg_statistics_result, rasters], 'Pick', {"ExtentType":2})

    def selectBands(self, band_ids_or_names):
        """
        Return a new raster collection that only contains the selected bands in each raster

        :param band_ids_or_names (List): a list of Band ID or Band Names. Band ID uses one-based indexing
        :return(RasterCollection): a raster collection with only the selected bands
        """

        # type checking
        by_bandID_or_bandName = 0  # 1: by band id; 2: by band name
        if not (isinstance(band_ids_or_names, list) or isinstance(band_ids_or_names, str) or isinstance(
                band_ids_or_names, int)):
            raise TypeError('bands must be either a list, a single band ID or a single band Name')
        if isinstance(band_ids_or_names, list):
            for band in band_ids_or_names:
                if not (isinstance(band, int) or isinstance(band, str)):
                    raise TypeError('elements in band_ids_or_names must be integer or string type')
                if isinstance(band, int) and by_bandID_or_bandName == 0:
                    by_bandID_or_bandName = 1
                elif isinstance(band, int) and by_bandID_or_bandName == 2:
                    raise TypeError('elements in band_ids_or_names should either all be integer or string type')
                if isinstance(band, str) and by_bandID_or_bandName == 0:
                    by_bandID_or_bandName = 2
                elif isinstance(band, str) and by_bandID_or_bandName == 1:
                    raise TypeError('elements in band_ids_or_names should either all be integer or string type')

        elif isinstance(band_ids_or_names, str):
            by_bandID_or_bandName = 2
        else:
            by_bandID_or_bandName = 1

        if 'Raster' in self.fields:
            rasters = self.getFieldValues('Raster')
        elif 'Path' in self.fields:
            rasters = [arcpy.Raster(path) for path in self.getFieldValues('Raster')]
        else:
            raise ValueError('invalid raster collection')

        assert by_bandID_or_bandName == 1 or by_bandID_or_bandName == 2

        new_rasters = []
        if by_bandID_or_bandName == 1:
            new_rasters = [Apply(raster, 'ExtractBand', {'BandIDs': band_ids_or_names}) for raster in rasters]
        elif by_bandID_or_bandName == 2:
            new_rasters = [Apply(raster, 'ExtractBand', {'BandNames': band_ids_or_names}) for raster in rasters]

        in_raster_collection = {}
        for field_name in self.fields:
            if field_name == 'Raster' or field_name == 'Path':
                continue
            try:
                # we could not getFieldValues from the field URI in mosaic dataset
                in_raster_collection[field_name] = self.getFieldValues(field_name)
            except:
                if self._is_mosaic and field_name == "Uri":
                    continue
                else:
                    raise RuntimeError('invalid field values in ' + str(field_name))

        return RasterCollection(new_rasters, in_raster_collection)

    def map(self, func):
        """
        apply a function to each item in the raster collection
        :param func: follows the signature:
                    Args: item, an item in the raster collection
                    Returns: dict, {'raster': arcpy.Raster, fieldName:fieldValue ...}
        :return:
            a new raster collection object that has applied the func to each item
        """
        res = map(func, iter(self))

        rasters = []
        attribute_dict = defaultdict(list)
        for item in res:
            rasters.append(item['raster'])
            for key, value in item.items():
                if key != 'raster':
                    attribute_dict[key].append(value)

        return RasterCollection(rasters, attribute_dict)

    def filterByRasterProperty(self, property_name, operator, property_values):
        """
        filter the raster collection by property of each raster

        :param property_name: str, the property name to operate on
        :param operator: "EQUALS", "LESS_THAN", "GREATER_THAN","NOT_EQUALS", "NOT_LESS_THAN", "NOT_GREATER_THAN",
                        "STARTS_WITH", "ENDS_WITH", "NOT_STARTS_WITH", "NOT_ENDS_WITH", "CONTAINS", "NOT_CONTAINS",
                        "IN" and "NOT_IN"
        :param property_values: String or List of String
        :return: filtered RasterCollection or None if no raster is selected
        """
        if self._is_mosaic:
            raster_query = self._build_query_string(property_name, operator.lower(), property_values)
            return self.filter(raster_query=raster_query)

        values = property_values
        if not isinstance(values, numbers.Number) and not isinstance(values, str) and not isinstance(values, list):
            raise TypeError('property_values must be number, string or list')

        if isinstance(values, list):
            for v in values:
                if not isinstance(values, numbers.Number) and not isinstance(v, str):
                    raise TypeError('property_values must be a list of number or a list of string')

        filtered_rasters = []
        attribute_dict = defaultdict(list)

        for item in iter(self):
            raster = self._get_raster_from_item(item)

            raster_properties = raster.properties
            if property_name not in raster_properties:
                continue
            property_value = raster.getProperty(property_name)
            operator = operator.lower()
            selected = False
            if operator == 'equals':
                selected = (property_value == values)
            elif operator == 'less_than':
                selected = (property_value < values)
            elif operator == 'greater_than':
                selected = (property_value > values)
            elif operator == 'not_equals':
                selected = (property_value != values)
            elif operator == 'not_less_than':
                selected = (property_value >= values)
            elif operator == 'not_greater_than':
                selected = (property_value <= values)
            elif operator == 'starts_with':
                selected = (property_value.startswith(values))
            elif operator == 'ends_with':
                selected = (property_value.endswith(values))
            elif operator == 'not_starts_with':
                selected = (not property_value.startswith(values))
            elif operator == 'not_ends_with':
                selected = (not property_value.endswith(values))
            elif operator == 'contains':
                selected = (values in property_value)
            elif operator == 'not_contains':
                selected = (values not in property_value)
            elif operator == 'in':
                if not isinstance(values, list):
                    raise TypeError("Invalid values. Note that values must be a list when operator is 'in'")
                selected = (property_value in values)
            elif operator == 'not_in':
                if not isinstance(values, list):
                    raise TypeError("Invalid values. Note that values must be a list when operator is 'not_in'")
                selected = (property_value not in values)
            else:
                raise ValueError('invalid operator')

            if selected:
                filtered_rasters.append(raster)
                for key, value in item.items():
                    if key not in ['Raster', 'Path']:
                        attribute_dict[key].append(value)

        if not filtered_rasters:
            warnings.warn("Warning: the output is None because no items have raster properties satisfying the query")
        return RasterCollection(filtered_rasters, attribute_dict) if filtered_rasters else None

    def filterByCalendarRange(self, calendar_field, start, end=None, time_field_name='StdTime', date_time_format=None):
        """
        filter the raster collection by a calindar_field and its start and end value (inclusive). i.e. if you would like
        to select all the rasters that have the time stamp on Monday, specify calendar_field as 'DAY_OF_WEEK' and put start and
        end to 1.

        :param calendar_field: string, one of 'YEAR', 'MONTH', 'QUARTER', 'WEEK_OF_YEAR', 'DAY_OF_YEAR', 'DAY_OF_MONTH',
         'DAY_OF_WEEK', 'HOUR'
        :param start: integer, the start time. inclusive.
        :param end: integer, default is None, if default is used, the end is set equal to start. inclusive.
        :param time_field_name: string, the time field anme, default is 'StdTime'.
        :param date_time_format: the time format that is used to format the time field values. Please ref the python
                                date time standard for this argument. https://docs.python.org/3/library/datetime.html#strftime-and-strptime-behavior
                                Default is None and this means using the Pro standard time format '%Y-%m-%dT%H:%M:%S'
                                and ignoring the following sub-second.

        :return: a filtered raster collection.
        """
        # validation
        calendar_field_types = ['YEAR', 'MONTH', 'QUARTER', 'WEEK_OF_YEAR', 'DAY_OF_YEAR', 'DAY_OF_MONTH',
                                'DAY_OF_WEEK', 'HOUR']
        if not isinstance(calendar_field, str):
            raise TypeError('calender_field must be string type')

        if calendar_field not in calendar_field_types:
            raise ValueError('invalid calender_field, must be one of ' + ', '.join(calendar_field_types))

        if time_field_name not in self.fields:
            raise ValueError('the time_field_name does not exist. Please input a valid name for time field.')

        if end is None:
            end = start
        # validate start and end type
        if not isinstance(start, int) or not isinstance(end, int):
            raise TypeError('start and end must be numeric.')
        if end < start:
            raise ValueError('end must be equal or larger than start.')
        # validate start and end value
        if calendar_field == 'MONTH':
            if start < 1 or start > 12 or end < 1 or end > 12:
                raise ValueError('start and end must be between [1, 12] for MONTH filter.')
        elif calendar_field == 'QUARTER':
            if start < 1 or start > 4 or end < 1 or end > 4:
                raise ValueError('start and end must be between [1, 4] for QUARTER filter.')
        elif calendar_field == 'WEEK_OF_YEAR':
            if start < 1 or start > 53 or end < 1 or end > 53:
                raise ValueError('start and end must be between [1, 53] for WEEK_OF_YEAR filter.')
        elif calendar_field == 'DAY_OF_YEAR':
            if start < 1 or start > 366 or end < 1 or end > 366:
                raise ValueError('start and end must be between [1, 366] for DAY_OF_YEAR filter.')
        elif calendar_field == 'DAY_OF_MONTH':
            if start < 1 or start > 31 or end < 1 or end > 31:
                raise ValueError('start and end must be between [1, 31] for DAY_OF_MONTH filter.')
        elif calendar_field == 'DAY_OF_WEEK':
            if start < 1 or start > 7 or end < 1 or end > 7:
                raise ValueError('start and end must be between [1, 7] for DAY_OF_WEEK filter.')
        elif calendar_field == 'HOUR':
            if start < 1 or start > 24 or end < 1 or end > 24:
                raise ValueError('start and end must be between [1, 24] for HOUR filter.')

        filtered_rasters = []
        attribute_dict = defaultdict(list)
        for item in iter(self):
            raster = self._get_raster_from_item(item)

            date_time = convert_string_to_date_time(item[time_field_name], date_time_format)

            selected = False
            if calendar_field == 'YEAR':
                if start <= date_time.year <= end:
                    selected = True
            elif calendar_field == 'MONTH':
                if start <= date_time.month <= end:
                    selected = True
            elif calendar_field == 'QUARTER':
                if start - 1 <= (date_time.month - 1) // 3 <= end - 1:
                    selected = True
            elif calendar_field == 'WEEK_OF_YEAR':
                week_number = int(date_time.strftime('%U'))
                if start <= week_number + 1 <= end:
                    selected = True
            elif calendar_field == 'DAY_OF_YEAR':
                yday = date_time.timetuple().tm_yday
                if start <= yday <= end:
                    selected = True
            elif calendar_field == 'DAY_OF_MONTH':
                mday = date_time.timetuple().tm_mday
                if start <= mday <= end:
                    selected = True
            elif calendar_field == 'DAY_OF_WEEK':
                wday = date_time.isoweekday()
                if start <= wday % 7 + 1 <= end:
                    selected = True
            elif calendar_field == 'HOUR':
                hour = date_time.timetuple().tm_hour
                if start <= hour <= end:
                    selected = True

            if selected is True:
                filtered_rasters.append(raster)
                for key, value in item.items():
                    if key not in ['Raster', 'Path']:
                        attribute_dict[key].append(value)

        if not filtered_rasters:
            warnings.warn(
                "Warning: the output is None because no items have raster properties satisfying the query")

        return RasterCollection(filtered_rasters, attribute_dict) if filtered_rasters else None

    def summarizeField(self, field_name, summary_type="ALL"):
        """
        Summarizes a numeric field of the RasterCollection based on the specified summary_type
        :param field_name: str, the field name to be summarized
        :param summary_type: str or list of str representing the summary type."COUNT", "COUNT_DISTINCT", "FIRST","HISTOGRAM", "MAX", "MEAN", "MIN",
                        "PRODUCT", "SAMPLE_SD", "SAMPLE_VAR", "SUM", "TOTAL_SD", "TOTAL_VAR", "ALL".
        :return: a dictionary with key being the summary type and the value being the summary value.
        """
        property_values = self.getFieldValues(field_name)
        summary_dict = {}
        import numpy as np
        import numbers

        if not isinstance(summary_type, list):
            summary_type = [summary_type]

        if "ALL" in map(str.upper, summary_type):
            summary_type = ["COUNT", "COUNT_DISTINCT", "FIRST", "HISTOGRAM", "MAX", "MEAN", "MIN",
                        "PRODUCT", "SAMPLE_SD", "SAMPLE_VAR", "SUM", "TOTAL_SD", "TOTAL_VAR"]

        from operator import is_not
        from functools import partial
        property_values_not_none = list(filter(partial(is_not, None), property_values))

        all_num = all(isinstance(x, numbers.Number) for x in property_values_not_none)
        if not all_num:
            raise RuntimeError("Only numeric fields can be summarized")
        try:
            for summary in summary_type:
                val = None
                summary = summary.lower()
                if summary == 'count':
                    val = len(property_values_not_none)

                elif summary == 'count_distinct':
                    val = len(np.unique(property_values_not_none))

                elif summary == 'first':
                    val = property_values_not_none[0]

                elif summary == 'histogram':
                    unique, counts = np.unique(property_values_not_none, return_counts=True)
                    val = dict(zip(unique, counts))

                elif summary == 'max':
                    val = np.max(property_values_not_none)

                elif summary == 'mean':
                    val = np.mean(property_values_not_none)

                elif summary == 'min':
                    val = np.min(property_values_not_none)

                elif summary == 'product':
                    val = np.prod(property_values_not_none)

                elif summary == 'sample_sd':
                    val = np.std(property_values_not_none, ddof=1)

                elif summary == 'sample_var':
                    val = np.var(property_values_not_none, ddof=1)

                elif summary == 'sum':
                    val = np.sum(property_values_not_none)

                elif summary == 'total_sd':
                    val = np.std(property_values_not_none)

                elif summary == 'total_var':
                    val = np.var(property_values_not_none)

                else:
                    raise ValueError('invalid summary_type value')

                summary_dict.update({summary:val})
        except:
            raise RuntimeError("Failed to summarize the property")

        return summary_dict
 
    def merge(self, collection2):
        """
         merge method merges two image collections into one. The output has all the items that were in either collection.
        :param collection2: RasterCollection object. The second collection to merge.
        :return: Collection that has all the items that were in either collection.
        """
        import pandas as pd

        rc1=self._export_data_frame()
        rc2=collection2._export_data_frame()

        merged_collection = pd.concat([rc1, rc2], axis=0)
        merged_collection.reset_index(drop=True, inplace=True)

        return RasterCollection(merged_collection)      
        
    def reduce(self, func, func_args={}): 
        """
        The ``reduce`` method composite all the images in the collection to a single image based on a reducer function. 

        :param func: The Python function to reduce the raster collection. 
                     The function should accept a list of rasters as first parameter and return a single raster
        :param func_args: Optional dictionary. Additional paramters to be passed the reducer function.
        :return: Returns what the the function specified in the func arg returns

        .. code-block:: python

            # Usage Example 1: This snippet reduces a raster collection based on a reducer function from arcpy.ia module that can accept a list of rasters

            rc = RasterCollection("<data path>")
            from arcpy.ia import Max
            max_raster = rc.reduce(func=Max, func_args={"extent_type":"UnionOf"})


            # Usage Example 2: This snippet reduces a raster collection based on a custom reducer function.

            rc = RasterCollection("<data path>")

            def skewness(ras_list):
                from arcpy.ia import CellStatistics, Power, Plus
                cs_mean = CellStatistics(ras_list, "MEAN", process_as_multiband="MULTI_BAND")
                cs_stddev  = CellStatistics(ras_list, "STD", process_as_multiband="MULTI_BAND")
                cs_median = CellStatistics(ras_list, "MEDIAN", process_as_multiband="MULTI_BAND")
                out_skewness = 3*(cs_mean - cs_median)/cs_stddev
                return out_skewness

            skewness = rc.reduce(func=skewness)

        """

        reduced_raster = func(self, **func_args)
        return reduced_raster


    def addField(self, field_name, field_values):
        """
         Adds a new field to the raster collection and populate it with values.
        :param field_name: Required string. The name of the field to be added. 
        :param field_values: Required list. The list of values associated with the field name. 
                             The length of the list should match the number of items in the raster collection
                             Providing only one value will set the same value for all rows. 
        :return: Collection that has the new field added.
        """

        if field_name in self.fields:
            raise RuntimeError('Cannot add the field. The field name already exists.')

        df=self._export_data_frame()
        new_df = df.copy()

        if not isinstance(field_values, list):
            field_values = [field_values]

        if self.count != len(field_values):
            if len(field_values) == 1:
                field_values = field_values*self.count
            else:                 
                raise RuntimeError('Length of field_values does not match the raster collection count')

        try:
            new_df[field_name] = field_values
        except:
            raise RuntimeError('Failed to add the field to the raster collection')

        return RasterCollection(new_df)

    def groupBy(self, field_name):
        """
         groupBy method can be used to group the raster collection based on a field.
        :param field_name: Required string.The name of the field that is used to group the raster collection. Items with the same field values will be grouped together. 
        :return: Dictionary.The dictionary that contains the grouped raster collections. The key of the dictionary is a field value of the field name that the grouping is based on. 
                 The value of the dictionary is a raster collection whose field name contains the same field value.
        """

        df = self._export_data_frame()
        try:
            group_by_obj = df.groupby(field_name)
            groups={}

            for key,val in group_by_obj.groups.items():
                groups.update({key:RasterCollection(group_by_obj.get_group(key))})
            return groups
        except:
            raise RuntimeError('groupBy failed with the field_name - '+field_name)



    def _get_raster_from_item(self, item):
        if 'Raster' in item:
            raster = item['Raster']
        elif 'Path' in item:
            raster = arcpy.Raster(item['Path'], is_multidimensional=True)
        else:
            raise RuntimeError('invalid raster collection')
        return raster
