# -*- coding: utf-8 -*-
#COPYRIGHT 2020 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The ArcGIS Indoor Positioning toolbox contains tools to create an
ArcGIS IPS Information Model, which enables indoor surveys with the
ArcGIS IPS Setup app, processing of survey data, and indoor
positioning. The toolbox also includes the Quality Assessment toolset
that contains tools to estimate the performance of an ArcGIS IPS
deployment."""
__all__ = ['ComputePositioningQuality', 'CreateIPSQualityDataset',
           'EnableIndoorPositioning', 'GenerateIndoorPositioningFile']
__alias__ = 'indoorpositioning'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptooldoc('EnableIndoorPositioning_indoorpositioning', None)
def EnableIndoorPositioning(in_workspace=None):
    """EnableIndoorPositioning_indoorpositioning(in_workspace)

        Creates the feature classes and table necessary for storing ArcGIS IPS
        data in an existing geodatabase.

     INPUTS:
      in_workspace (Workspace):
          The geodatabase where the ArcGIS IPS table and feature classes will be
          created. It can be a file or enterprise geodatabase."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EnableIndoorPositioning_indoorpositioning(*gp_fixargs((in_workspace,), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GenerateIndoorPositioningFile_indoorpositioning', None)
def GenerateIndoorPositioningFile(in_ips_recordings=None, target_ips_positioning=None, in_ips_transitions=None, in_ips_comment=None):
    """GenerateIndoorPositioningFile_indoorpositioning(in_ips_recordings, target_ips_positioning, {in_ips_transitions}, {in_ips_comment})

        Generates a positioning file from ArcGIS IPS Setup survey recordings.

     INPUTS:
      in_ips_recordings (Feature Layer):
          The feature class or feature service that contains IPS Setup survey
          recordings.
      target_ips_positioning (Table View):
          The table or feature service where the generated IPS positioning file
          will be stored.
      in_ips_transitions {Feature Layer}:
          The line feature class that contains the TRANSITION_TYPE,
          VERTICAL_ORDER_FROM, and VERTICAL_ORDER_TO fields that define facility
          entrances and exits. These are used by ArcGIS IPS to improve indoor
          and outdoor localization and switching. The TRANSITION_TYPE field for
          entrances and exits must contain a value of 7 to be used by this tool.
      in_ips_comment {String}:
          The text that will be used to populate the Comment field of the
          positioning file entry in the target_ips_positioning value."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateIndoorPositioningFile_indoorpositioning(*gp_fixargs((in_ips_recordings, target_ips_positioning, in_ips_transitions, in_ips_comment), True)))
        return retval
    except Exception as e:
        raise e


# Quality Assessment toolset
@gptooldoc('ComputePositioningQuality_indoorpositioning', None)
def ComputePositioningQuality(target_ips_quality_dataset=None, in_ips_recordings_features=None, in_sight_blocking_details_features=None):
    """ComputePositioningQuality_indoorpositioning(target_ips_quality_dataset, in_ips_recordings_features, {in_sight_blocking_details_features})

        Populates an IPS Quality dataset generated by the Create IPS Quality
        Dataset tool.

     INPUTS:
      target_ips_quality_dataset (Dataset):
          The IPS Quality dataset generated by the Create IPS Quality Dataset
          tool that will be populated.
      in_ips_recordings_features (Feature Layer):
          The IPS Recordings feature layer that contains recordings collected
          using the ArcGIS IPS Setup app.
      in_sight_blocking_details_features {Feature Layer}:
          Details features from an Indoors dataset representing obstacles for
          line-of-sight calculation. It is recommended that you only use
          meaningful features, such as features that represent walls."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ComputePositioningQuality_indoorpositioning(*gp_fixargs((target_ips_quality_dataset, in_ips_recordings_features, in_sight_blocking_details_features), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateIPSQualityDataset_indoorpositioning', None)
def CreateIPSQualityDataset(target_workspace=None, coordinate_system=None, out_dataset_name=None):
    """CreateIPSQualityDataset_indoorpositioning(target_workspace, coordinate_system, out_dataset_name)

        Creates a dataset that can be used to assess the quality of an ArcGIS
        IPS deployment.

     INPUTS:
      target_workspace (Workspace):
          The geodatabase where the ArcGIS IPS Quality dataset will be created.
          The value can be a file or an enterprise geodatabase.
      coordinate_system (Coordinate System):
          The spatial reference that will be used for the output IPS Quality
          dataset. The default is WGS84. You can specify the spatial reference
          in several ways, including the following:

          * Reference a feature class or feature dataset with the spatial
          reference you want to apply, such as
          C:/workspace/myproject.gdb/IPS_Recordings.

          * Use a SpatialReference object.

          * Use the well-known text (WKT) string of a spatial reference. One way
          to determine the WKT of a spatial reference is to export a
          SpatialReference object to a string using the exportToString method.
      out_dataset_name (String):
          The name of the dataset that will be created. The default is
          IPS_Quality."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateIPSQualityDataset_indoorpositioning(*gp_fixargs((target_workspace, coordinate_system, out_dataset_name), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject